# frozen_string_literal: true

require_relative 'api/client'
require_relative 'cheatsheet/printer'

module HintMe
  # entry point for all HintMe functionality
  class Hinter
    # creates a new {Hinter} instance
    #
    def initialize
      @api_client = API::Client.new
      @cheatsheet_printer = Cheatsheet::Printer.new
    end

    # searches and prints a cheatsheet for a given topic
    # to the console
    #
    # if no cheatsheet is found for the given topic
    # a error message will be printed to the console
    #
    # @param topic [String]
    #
    def hint(topic)
      cheatsheet = cheatsheet_for_topic(topic)

      if cheatsheet
        print_cheatsheet(cheatsheet)
      else
        display_missing_cheatsheet_error(topic)
      end
    end

    private

    # fetches the cheatsheet for a topic
    #
    # @param topic [String]
    # @return [String]
    #
    def cheatsheet_for_topic(topic)
      @api_client.search_cheatsheet_for_topic(topic)
    end

    # prints a cheatsheet to the console
    #
    # @param cheatsheet [String]
    #
    def print_cheatsheet(cheatsheet)
      @cheatsheet_printer.print(cheatsheet)
    end

    # displays a error message informing the user about
    # the missing cheatsheet to the console
    #
    def display_missing_cheatsheet_error(topic)
      topic ||= 'escaping the void of notopic-island'

      puts ":( There isn't a cheatsheet for #{topic}." \
           ' Maybe you should request one at devhints.io'
    end
  end
end
